// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024-2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/GuardGallivant.hpp>

const std::string GuardGallivant::getPuzzleName() const
{
    return "Day 6: Guard Gallivant";
}

const std::string GuardGallivant::getInputFileName() const
{
    return "guard_gallivant.txt";
}

void GuardGallivant::processDataLine(const std::string& line)
{
    LinesSolver::processDataLine(line);
    auto pos{ line.find(getStartChar()) };
    if (pos != std::string::npos)
    {
        start_ = Point2{ static_cast<int>(pos), static_cast<int>(lines.size() - 1) };
    }
}

void GuardGallivant::finish()
{
    auto dirIndex{ getStartDirectionIndex() };
    auto current{ start_ };
    visitPosition(current);
    auto next{ current + Point2::cardinalDirections[dirIndex] };
    while (isInBounds(next))
    {
        if (getCharAt(next) == getObstructionChar())
        {
            dirIndex = turnDirection(dirIndex);
        }
        else
        {
            current = next;
            visitPosition(current);
        }
        next = current + Point2::cardinalDirections[dirIndex];
    }
}

constexpr size_t GuardGallivant::getStartDirectionIndex()
{
    return 2;
}

constexpr char GuardGallivant::getStartChar()
{
    return '^';
}

constexpr char GuardGallivant::getVisitedChar()
{
    return 'X';
}

constexpr char GuardGallivant::getObstructionChar()
{
    return '#';
}

void GuardGallivant::visitPosition(const Point2& current)
{
    if (getCharAt(current) != getVisitedChar())
    {
        setCharAt(current, getVisitedChar());
        part1++;
    }
}

size_t GuardGallivant::turnDirection(const size_t current) const
{
    return current == 0 ? 3 : current - 1;
}
