// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024-2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <catch2/catch_test_macros.hpp>

#include <aoc/HistorianHysteria.hpp>
#include <aoc/RedNosedReports.hpp>
#include <aoc/MullItOver.hpp>
#include <aoc/CeresSearch.hpp>
#include <aoc/PrintQueue.hpp>
#include <aoc/GuardGallivant.hpp>
#include <aoc/BridgeRepair.hpp>
#include <aoc/ResonantCollinearity.hpp>
#include <aoc/DiskFragmenter.hpp>
#include <aoc/HoofIt.hpp>
#include <aoc/PlutonianPebbles.hpp>
#include <aoc/LanParty.hpp>
#include <aocTests/TestContext.hpp>

#define REQUIRE_MESSAGE(cond, msg) if (!(cond)) { INFO(msg); REQUIRE(cond); }

TEST_CASE("[HistorianHysteriaTests]")
{
    TestContext test;
    SECTION("FullData")
    {
        test.run(std::make_unique<HistorianHysteria>(), 2176849, 23384288, test.getInputPaths());
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<HistorianHysteria>(), 11, 31, test.getExampleInputPaths());
    }
}

TEST_CASE("[RedNosedReportsTests]")
{
    TestContext test;
    SECTION("FullData")
    {
        test.run(std::make_unique<RedNosedReports>(), 472, 520, test.getInputPaths());
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<RedNosedReports>(), 2, 4, test.getExampleInputPaths());
    }
}

TEST_CASE("[MullItOverTests]")
{
    TestContext test;
    SECTION("FullData")
    {
        test.run(std::make_unique<MullItOver>(), 187833789, 94455185, test.getInputPaths());
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<MullItOver>(), 161, 48, test.getExampleInputPaths());
    }
}

TEST_CASE("[CeresSearchTests]")
{
    TestContext test;
    SECTION("FullData")
    {
        test.run(std::make_unique<CeresSearch>(), 2462, 1877, test.getInputPaths());
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<CeresSearch>(), 18, 9, test.getExampleInputPaths());
    }
}

TEST_CASE("[PrintQueueTests]")
{
    TestContext test;
    SECTION("FullData")
    {
        test.run(std::make_unique<PrintQueue>(), 5087, 4971, test.getInputPaths());
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<PrintQueue>(), 143, 123, test.getExampleInputPaths());
    }
}

TEST_CASE("[GuardGallivantTests]")
{
    TestContext test;
    SECTION("FullData")
    {
        test.run(std::make_unique<GuardGallivant>(), 4665, 0, test.getInputPaths());
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<GuardGallivant>(), 41, 0, test.getExampleInputPaths());
    }
}

TEST_CASE("[BridgeRepairTests]")
{
    TestContext test;
    SECTION("FullData")
    {
        test.run(std::make_unique<BridgeRepair>(), 12839601725877, 149956401519484, test.getInputPaths());
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<BridgeRepair>(), 3749, 11387, test.getExampleInputPaths());
    }
}

TEST_CASE("[ResonantCollinearityTests]")
{
    TestContext test;
    SECTION("FullData")
    {
        test.run(std::make_unique<ResonantCollinearity>(), 426, 1359, test.getInputPaths());
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<ResonantCollinearity>(), 14, 34, test.getExampleInputPaths());
    }
}

TEST_CASE("[DiskFragmenterTests]")
{
    TestContext test;
    SECTION("FullData")
    {
        test.run(std::make_unique<DiskFragmenter>(), 6401092019345, 0, test.getInputPaths());
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<DiskFragmenter>(), 1928, 0, test.getExampleInputPaths());
    }
}

TEST_CASE("[HoofItTests]")
{
    TestContext test;
    SECTION("FullData")
    {
        test.run(std::make_unique<HoofIt>(), 611, 1380, test.getInputPaths());
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<HoofIt>(), 36, 81, test.getExampleInputPaths());
    }
}

TEST_CASE("[PlutonianPebblesTests]")
{
    TestContext test;
    SECTION("FullData")
    {
        test.run(std::make_unique<PlutonianPebbles>(), 220999, 261936432123724, test.getInputPaths());
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<PlutonianPebbles>(), 55312, 65601038650482, test.getExampleInputPaths());
    }
}

TEST_CASE("[LanPartyTests]")
{
    TestContext test;
    SECTION("FullData")
    {
        test.run(std::make_unique<LanParty>(), 1230, 0, test.getInputPaths());
    }
    SECTION("ExampleData")
    {
        test.run(std::make_unique<LanParty>(), 7, 0, test.getExampleInputPaths());
    }
}
